class_name StateMachine extends Node

@export var initial_state: State
@export var anim_player: AnimationPlayer

var states: Dictionary = {}
var current_state: State


func init(entity):
	for child in get_children():
		if child is State:
			states[child.name.to_lower()] = child
			child.transitioned.connect(on_child_transition)
			child.entity      = entity
			child.anim_player = anim_player

	if initial_state:
		initial_state.enter()
		current_state = initial_state


func process(delta):
	if current_state:
		current_state.update(delta)

func physics_process(delta):
	if current_state:
		current_state.physics_update(delta)

func on_child_transition(state, new_state_name):
	if state != current_state:
		return

	var new_state = states.get(new_state_name.to_lower())
	if !new_state:
		return

	if current_state:
		current_state.exit()

	new_state.enter()
	current_state = new_state
